// ----------------------------------------------
#include <assert.h>
// ----------------------------------------------
#ifndef GPUMESH_NO_GLUX
#  include <glux.h>
#endif
// ----------------------------------------------
#include "VertexStorage.h"
#include "CGPUmesh_GL_types.h"
#include "CGPUmesh_GL_funcs.h"
// ----------------------------------------------
namespace GPUmesh {
  // ----------------------------------------------
  template <class VertexFormat,class T_IndexType> 
  class CompilePolicy_GL_DisplayList
  {
  public:

    typedef struct { unsigned char data[MVF_sizeof<VertexFormat>::value]; } vertex_data;
    typedef FormatDescriptor<VertexFormat>                                  vertex_format_desc;
    typedef VertexStorageInterlaced<VertexFormat,vertex_data>               vertex_storage;
    typedef std::vector<T_IndexType>                                        index_storage;

    enum {nb_vertex_attrib = Loki::TL::Length<VertexFormat>::value};

    typedef struct
    {
      GLuint uiDP;
    }mesh_descriptor;

    static void     add_vertex(const vertex_storage& storage,int i)
    {
      assert(i >= 0 && i < (int)storage.m_Vertices.size());

      if (vertex_format_desc::has_normal)
      {
        (GLFuncs<typename vertex_format_desc::type_normal::type>::getNormalFct(vertex_format_desc::type_normal::components - 1))
          ((typename vertex_format_desc::type_normal::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_normal]));
      }
      if (vertex_format_desc::has_color0)
      {
        (GLFuncs<typename vertex_format_desc::type_color0::type>::getColor0Fct(vertex_format_desc::type_color0::components - 1))
          ((typename vertex_format_desc::type_color0::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_color0]));
      }
      if (vertex_format_desc::has_color1)
      {
        (GLFuncs<typename vertex_format_desc::type_color1::type>::getAttributeFct(vertex_format_desc::type_color1::components - 1))
          (4,(typename vertex_format_desc::type_color1::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_color1]));
      }

      if (vertex_format_desc::has_texcoord0)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord0::type>::getTexCoordnFct(vertex_format_desc::type_texcoord0::components - 1))
          (GL_TEXTURE0_ARB,(typename vertex_format_desc::type_texcoord0::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord0]));
      }

      if (vertex_format_desc::has_texcoord1)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord1::type>::getTexCoordnFct(vertex_format_desc::type_texcoord1::components - 1))
          (GL_TEXTURE1_ARB,(typename vertex_format_desc::type_texcoord1::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord1]));
      }

      if (vertex_format_desc::has_texcoord2)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord2::type>::getTexCoordnFct(vertex_format_desc::type_texcoord2::components - 1))
          (GL_TEXTURE2_ARB,(typename vertex_format_desc::type_texcoord2::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord2]));
      }

      if (vertex_format_desc::has_texcoord3)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord3::type>::getTexCoordnFct(vertex_format_desc::type_texcoord3::components - 1))
          (GL_TEXTURE3_ARB,(typename vertex_format_desc::type_texcoord3::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord3]));
      }

      if (vertex_format_desc::has_texcoord4)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord4::type>::getTexCoordnFct(vertex_format_desc::type_texcoord4::components - 1))
          (GL_TEXTURE4_ARB,(typename vertex_format_desc::type_texcoord4::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord4]));
      }

      if (vertex_format_desc::has_texcoord5)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord5::type>::getTexCoordnFct(vertex_format_desc::type_texcoord5::components - 1))
          (GL_TEXTURE5_ARB,(typename vertex_format_desc::type_texcoord5::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord5]));
      }

      if (vertex_format_desc::has_texcoord6)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord6::type>::getTexCoordnFct(vertex_format_desc::type_texcoord6::components - 1))
          (GL_TEXTURE6_ARB,(typename vertex_format_desc::type_texcoord6::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord6]));
      }

      if (vertex_format_desc::has_texcoord7)
      {
        (GLFuncs<typename vertex_format_desc::type_texcoord7::type>::getTexCoordnFct(vertex_format_desc::type_texcoord7::components - 1))
          (GL_TEXTURE7_ARB,(typename vertex_format_desc::type_texcoord7::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_texcoord7]));
      }

      if (vertex_format_desc::has_vertex)
      {
        (GLFuncs<typename vertex_format_desc::type_vertex::type>::getVertexFct(vertex_format_desc::type_vertex::components - 1))
          ((typename vertex_format_desc::type_vertex::type *)&(storage.m_Vertices[i].data[vertex_format_desc::offset_vertex]));
      }
    }

    // ---------------------------------

    static void compile(int primtype,
      vertex_storage& storage,
      index_storage&  indices,
      mesh_descriptor& _mesh)
    {
      // create display list
      _mesh.uiDP=glGenLists(1);
      glNewList(_mesh.uiDP,GL_COMPILE);
      glBegin(primtype);
      for (unsigned int i=0;i<storage.m_Vertices.size();i++)
      {
        add_vertex(storage,i);
      }
      glEnd();
      glEndList();

      // free memory
      storage.clear();
      indices.clear();
    }

    static void     bind(const mesh_descriptor& m)
    {
    }

    static void     unbind(const mesh_descriptor& m)
    {
    }

    static void     free(const mesh_descriptor& m)
    {
      glDeleteLists(m.uiDP,1);
    }

    static void     draw(const mesh_descriptor& m)
    {
      glCallList(m.uiDP);
    }
  };

  // ----------------------------------------------
  // Indexed mesh
  // ----------------------------------------------

  template <class T_VertexFormat,class T_IndexType> 
  class CompilePolicy_Indexed_GL_DisplayList : public CompilePolicy_GL_DisplayList<T_VertexFormat,T_IndexType>
  {
  public:
    typedef CompilePolicy_GL_DisplayList<T_VertexFormat,T_IndexType> parent_policy;
    typedef typename parent_policy::vertex_storage  vertex_storage;
    typedef typename parent_policy::index_storage   index_storage;
    typedef typename parent_policy::mesh_descriptor mesh_descriptor;
  public:
    static void compile(int primtype,
      vertex_storage& storage,
      index_storage&  indices,
      mesh_descriptor& _mesh)
    {
      // create display list
      _mesh.uiDP=glGenLists(1);
      glNewList(_mesh.uiDP,GL_COMPILE);
      glBegin(primtype);
      int nb=storage.m_Vertices.size();
      for (unsigned int i=0;i<indices.size();i++)
      {
        add_vertex(storage,indices[i]);
      }
      glEnd();
      glEndList();

      // free memory
      storage.clear();
      indices.clear();
    }
  };
  // ----------------------------------------------
  // define class with this compile policy
  // ----------------------------------------------
  template <class VertexFormat> 
  class GPUmesh_GL_DisplayList
    : public GPUmesh_generic< CompilePolicy_GL_DisplayList , VertexFormat > {};

  template <class T_VertexFormat,class T_IndexType> 
  class GPUmesh_Indexed_GL_DisplayList
    : public GPUmesh_generic< CompilePolicy_Indexed_GL_DisplayList , T_VertexFormat , T_IndexType > {};
  // ----------------------------------------------
} // namespace GPUmesh
// ----------------------------------------------
