
// Base attributes
class MVF_BASE_VERTEX    {};
class MVF_BASE_NORMAL    {};
class MVF_BASE_COLOR0    {};
class MVF_BASE_COLOR1    {};
class MVF_BASE_TEXCOORD0 {};
class MVF_BASE_TEXCOORD1 {};
class MVF_BASE_TEXCOORD2 {};
class MVF_BASE_TEXCOORD3 {};
class MVF_BASE_TEXCOORD4 {};
class MVF_BASE_TEXCOORD5 {};
class MVF_BASE_TEXCOORD6 {};
class MVF_BASE_TEXCOORD7 {};

// Attribute template
template <class Base,class StorageType,unsigned int sz> 
class MVF_ITEM
{
public:
  typedef Base        base_attribute;
  typedef StorageType type;
  enum               {size_of = sizeof(StorageType)*sz};
  enum               {components = sz};

  StorageType         data[sz];
};

// Empty attribute
typedef char MVF_null;
class   MVF_empty : public MVF_ITEM<Loki::NullType,MVF_null,1> {};

// Attributes
template <class StorageType,unsigned int sz> 
  class MVF_VERTEX : public MVF_ITEM<MVF_BASE_VERTEX,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_NORMAL : public MVF_ITEM<MVF_BASE_NORMAL,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_COLOR0 : public MVF_ITEM<MVF_BASE_COLOR0,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_COLOR1 : public MVF_ITEM<MVF_BASE_COLOR1,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD0 : public MVF_ITEM<MVF_BASE_TEXCOORD0,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD1 : public MVF_ITEM<MVF_BASE_TEXCOORD1,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD2 : public MVF_ITEM<MVF_BASE_TEXCOORD2,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD3 : public MVF_ITEM<MVF_BASE_TEXCOORD3,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD4 : public MVF_ITEM<MVF_BASE_TEXCOORD4,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD5 : public MVF_ITEM<MVF_BASE_TEXCOORD5,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD6 : public MVF_ITEM<MVF_BASE_TEXCOORD6,StorageType,sz> {};
template <class StorageType,unsigned int sz> 
  class MVF_TEXCOORD7 : public MVF_ITEM<MVF_BASE_TEXCOORD7,StorageType,sz> {};

// -------------------------------------------------
// typedef some standard attributes
// -------------------------------------------------

typedef MVF_VERTEX<float,2>         mvf_vertex_2f;
typedef MVF_VERTEX<double,2>        mvf_vertex_2d;
typedef MVF_VERTEX<float,3>         mvf_vertex_3f;
typedef MVF_VERTEX<double,3>        mvf_vertex_3d;
typedef MVF_VERTEX<float,4>         mvf_vertex_4f;
typedef MVF_VERTEX<double,4>        mvf_vertex_4d;
typedef MVF_NORMAL<float,3>         mvf_normal_3f;
typedef MVF_NORMAL<double,3>        mvf_normal_3d;
typedef MVF_COLOR0<float,3>         mvf_color0_3f;
typedef MVF_COLOR0<double,3>        mvf_color0_3d;
typedef MVF_COLOR0<float,4>         mvf_color0_4f;
typedef MVF_COLOR0<double,4>        mvf_color0_4d;
typedef MVF_COLOR0<unsigned char,4> mvf_color0_rgba;
typedef MVF_COLOR0<unsigned char,3> mvf_color0_rgb;
typedef MVF_COLOR1<float,3>         mvf_color1_3f;
typedef MVF_COLOR1<double,3>        mvf_color1_3d;
typedef MVF_COLOR1<float,4>         mvf_color1_4f;
typedef MVF_COLOR1<double,4>        mvf_color1_4d;
typedef MVF_COLOR1<unsigned char,4> mvf_color1_rgba;
typedef MVF_COLOR1<unsigned char,3> mvf_color1_rgb;
typedef MVF_TEXCOORD0<float,1>      mvf_texcoord0_1f;
typedef MVF_TEXCOORD0<double,1>     mvf_texcoord0_1d;
typedef MVF_TEXCOORD0<float,2>      mvf_texcoord0_2f;
typedef MVF_TEXCOORD0<double,2>     mvf_texcoord0_2d;
typedef MVF_TEXCOORD0<float,3>      mvf_texcoord0_3f;
typedef MVF_TEXCOORD0<double,3>     mvf_texcoord0_3d;
typedef MVF_TEXCOORD0<float,4>      mvf_texcoord0_4f;
typedef MVF_TEXCOORD0<double,4>     mvf_texcoord0_4d;
typedef MVF_TEXCOORD1<float,1>      mvf_texcoord1_1f;
typedef MVF_TEXCOORD1<double,1>     mvf_texcoord1_1d;
typedef MVF_TEXCOORD1<float,2>      mvf_texcoord1_2f;
typedef MVF_TEXCOORD1<double,2>     mvf_texcoord1_2d;
typedef MVF_TEXCOORD1<float,3>      mvf_texcoord1_3f;
typedef MVF_TEXCOORD1<double,3>     mvf_texcoord1_3d;
typedef MVF_TEXCOORD1<float,4>      mvf_texcoord1_4f;
typedef MVF_TEXCOORD1<double,4>     mvf_texcoord1_4d;
typedef MVF_TEXCOORD2<float,1>      mvf_texcoord2_1f;
typedef MVF_TEXCOORD2<double,1>     mvf_texcoord2_1d;
typedef MVF_TEXCOORD2<float,2>      mvf_texcoord2_2f;
typedef MVF_TEXCOORD2<double,2>     mvf_texcoord2_2d;
typedef MVF_TEXCOORD2<float,3>      mvf_texcoord2_3f;
typedef MVF_TEXCOORD2<double,3>     mvf_texcoord2_3d;
typedef MVF_TEXCOORD2<float,4>      mvf_texcoord2_4f;
typedef MVF_TEXCOORD2<double,4>     mvf_texcoord2_4d;
typedef MVF_TEXCOORD3<float,1>      mvf_texcoord3_1f;
typedef MVF_TEXCOORD3<double,1>     mvf_texcoord3_1d;
typedef MVF_TEXCOORD3<float,2>      mvf_texcoord3_2f;
typedef MVF_TEXCOORD3<double,2>     mvf_texcoord3_2d;
typedef MVF_TEXCOORD3<float,3>      mvf_texcoord3_3f;
typedef MVF_TEXCOORD3<double,3>     mvf_texcoord3_3d;
typedef MVF_TEXCOORD3<float,4>      mvf_texcoord3_4f;
typedef MVF_TEXCOORD3<double,4>     mvf_texcoord3_4d;
typedef MVF_TEXCOORD4<float,1>      mvf_texcoord4_1f;
typedef MVF_TEXCOORD4<double,1>     mvf_texcoord4_1d;
typedef MVF_TEXCOORD4<float,2>      mvf_texcoord4_2f;
typedef MVF_TEXCOORD4<double,2>     mvf_texcoord4_2d;
typedef MVF_TEXCOORD4<float,3>      mvf_texcoord4_3f;
typedef MVF_TEXCOORD4<double,3>     mvf_texcoord4_3d;
typedef MVF_TEXCOORD4<float,4>      mvf_texcoord4_4f;
typedef MVF_TEXCOORD4<double,4>     mvf_texcoord4_4d;
typedef MVF_TEXCOORD5<float,1>      mvf_texcoord5_1f;
typedef MVF_TEXCOORD5<double,1>     mvf_texcoord5_1d;
typedef MVF_TEXCOORD5<float,2>      mvf_texcoord5_2f;
typedef MVF_TEXCOORD5<double,2>     mvf_texcoord5_2d;
typedef MVF_TEXCOORD5<float,3>      mvf_texcoord5_3f;
typedef MVF_TEXCOORD5<double,3>     mvf_texcoord5_3d;
typedef MVF_TEXCOORD5<float,4>      mvf_texcoord5_4f;
typedef MVF_TEXCOORD5<double,4>     mvf_texcoord5_4d;
typedef MVF_TEXCOORD6<float,1>      mvf_texcoord6_1f;
typedef MVF_TEXCOORD6<double,1>     mvf_texcoord6_1d;
typedef MVF_TEXCOORD6<float,2>      mvf_texcoord6_2f;
typedef MVF_TEXCOORD6<double,2>     mvf_texcoord6_2d;
typedef MVF_TEXCOORD6<float,3>      mvf_texcoord6_3f;
typedef MVF_TEXCOORD6<double,3>     mvf_texcoord6_3d;
typedef MVF_TEXCOORD6<float,4>      mvf_texcoord6_4f;
typedef MVF_TEXCOORD6<double,4>     mvf_texcoord6_4d;
typedef MVF_TEXCOORD7<float,1>      mvf_texcoord7_1f;
typedef MVF_TEXCOORD7<double,1>     mvf_texcoord7_1d;
typedef MVF_TEXCOORD7<float,2>      mvf_texcoord7_2f;
typedef MVF_TEXCOORD7<double,2>     mvf_texcoord7_2d;
typedef MVF_TEXCOORD7<float,3>      mvf_texcoord7_3f;
typedef MVF_TEXCOORD7<double,3>     mvf_texcoord7_3d;
typedef MVF_TEXCOORD7<float,4>      mvf_texcoord7_4f;
typedef MVF_TEXCOORD7<double,4>     mvf_texcoord7_4d;
