// --------------------------------------------------------------
//
// OpenGL tutorial 1 of the GPUmesh class (indexed meshes)
//
// Dependencies
//
//    * glut
//    * gluX 
//
// (c) Sylvain Lefebvre 2003 - http://www.aracknea.net
//
//     Sylvain.Lefebvre@laposte.net
// --------------------------------------------------------------
#pragma warning(disable:4786)
/* -------------------------------------------------------- */
#ifdef WIN32
#include <windows.h>
#endif
/* -------------------------------------------------------- */
#include <cmath>
#include <GL/gl.h>
#include <GL/glut.h>
/* -------------------------------------------------------- */
#include "GLGPUmesh.h"
/* -------------------------------------------------------- */
using namespace GPUmesh;
/* -------------------------------------------------------- */

// define our vertex format

typedef GPUMESH_MVF2(mvf_vertex_2f,mvf_color0_rgba) mvf_mesh;

// associate mesh and choosen vertex format

typedef GPUmesh_Indexed_GL_VBO<mvf_mesh,unsigned int> IndexedMesh;

IndexedMesh         *g_Object=NULL;

/* -------------------------------------------------------- */

int     g_iMainWindow;        // window glut id
int     g_iWinWidth=640;
int     g_iWinHeight=480;

/* -------------------------------------------------------- */

void mainKeyboard(unsigned char key, int x, int y) 
{
  if (key == 'q')
    exit (0);
}

/* -------------------------------------------------------- */

void mainMouse(int btn, int state, int x, int y) 
{

}

/* -------------------------------------------------------- */

void mainMotion(int x,int y)
{

}

/* -------------------------------------------------------- */

void mainReshape(int w,int h)
{
  g_iWinWidth=w;
  g_iWinHeight=h;
  glutPostRedisplay();
}

/* -------------------------------------------------------- */

void mainIdle()
{
  glutPostRedisplay();
}

/* -------------------------------------------------------- */

void mainRender()
{
  glViewport(0,0,g_iWinWidth,g_iWinHeight);
	
  // clear screen
  glClearColor(0.5,0.5,0.5,0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0,1.0,1.0,0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // render object
  g_Object->render();

  // swap buffers
  glutSwapBuffers();
}
/* -------------------------------------------------------- */
int main(int argc, char **argv) 
{
  // GLUT
  glutInit(&argc, argv);
  glutInitWindowSize(g_iWinWidth,g_iWinHeight);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);

  // main window
  g_iMainWindow=glutCreateWindow("GPUmesh OpenGL tutorial 1 (indexed mesh)");
  glutMouseFunc(mainMouse);
  glutMotionFunc(mainMotion);
  glutKeyboardFunc(mainKeyboard);
  glutDisplayFunc(mainRender);
  glutReshapeFunc(mainReshape);
  glutSetWindow(g_iMainWindow);
  glutIdleFunc(mainIdle);

  // gluX
  gluxInit();

  // create objects

  g_Object = new IndexedMesh();

  g_Object->begin(GPUMESH_TRIANGLELIST);

  g_Object->color0_4(0,0,255,0);
  g_Object->vertex_2(0.1,0.1);

  g_Object->color0_4(255,0,0,0);
  g_Object->vertex_2(0.1,0.9);

  g_Object->color0_4(0,255,0,0);
  g_Object->vertex_2(0.9,0.9);

  g_Object->color0_4(64,0,255,0);
  g_Object->vertex_2(0.9,0.1);

  g_Object->index(0);
  g_Object->index(1);
  g_Object->index(2);

  g_Object->index(0);
  g_Object->index(2);
  g_Object->index(3);

  g_Object->end();

  // help
  printf("[q]     - quit\n");
	
  // gl init

  // let's go
  glutMainLoop();

  return (0);
}
/* -------------------------------------------------------- */
