// --------------------------------------------------------------
//
// OpenGL tutorial 2 of the GPUmesh class (indexed meshes)
//
// Dependencies
//
//    * glut
//    * gluX
//    * libTexture
//
// (c) Sylvain Lefebvre 2004 - http://www.aracknea.net
//
//     Sylvain.Lefebvre@laposte.net
// --------------------------------------------------------------
#pragma warning(disable:4786)
/* -------------------------------------------------------- */
#ifdef WIN32
#include <windows.h>
#endif
/* -------------------------------------------------------- */
#include <cmath>
#include <GL/gl.h>
#include <GL/glut.h>
#include <gltex.h>
#include <iostream>
using namespace std;
/* -------------------------------------------------------- */
#include <glux.h>
/* -------------------------------------------------------- */
#include "GLGPUmesh.h"

using namespace GPUmesh;
/* -------------------------------------------------------- */

// define our vertex format

typedef GPUMESH_MVF10(mvf_vertex_2d,
		      mvf_color0_rgba,
		      mvf_texcoord0_2d,
		      mvf_texcoord1_2d,
		      mvf_texcoord2_2d,
		      mvf_texcoord3_2d,
		      mvf_texcoord4_2d,
		      mvf_texcoord5_2d,
		      mvf_texcoord6_2d,
		      mvf_texcoord7_2d     ) mvf_mesh;

// associate mesh and choosen vertex format

typedef GPUmesh_Indexed_GL_DisplayList<mvf_mesh,unsigned int> IndexedMesh;

IndexedMesh         *g_Object=NULL;

/* -------------------------------------------------------- */

int     g_iMainWindow;        // window glut id
int     g_iWinWidth=640;
int     g_iWinHeight=480;

GLuint  g_Texs[8];
int     g_iNbTU;

/* -------------------------------------------------------- */

void mainKeyboard(unsigned char key, int x, int y) 
{
  if (key == 'q')
    exit (0);
}

/* -------------------------------------------------------- */

void mainMouse(int btn, int state, int x, int y) 
{

}

/* -------------------------------------------------------- */

void mainMotion(int x,int y)
{

}

/* -------------------------------------------------------- */

void mainReshape(int w,int h)
{
  g_iWinWidth=w;
  g_iWinHeight=h;
  glutPostRedisplay();
}

/* -------------------------------------------------------- */

void mainIdle()
{
  glutPostRedisplay();
}

/* -------------------------------------------------------- */

void mainRender()
{
  glViewport(0,0,g_iWinWidth,g_iWinHeight);
	
  // clear screen
  glClearColor(0.5,0.5,0.5,0.0);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0,1.0,1.0,0.0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  // setup textures
  for (int i=0;i<g_iNbTU;i++)
  {
    glActiveTextureARB(GL_TEXTURE0_ARB + i);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D,g_Texs[i]);
  }
  glActiveTextureARB(GL_TEXTURE0_ARB);

  // render object
  g_Object->render();

  // swap buffers
  glutSwapBuffers();
}
/* -------------------------------------------------------- */
int main(int argc, char **argv) 
{
  // GLUT
  glutInit(&argc, argv);
  glutInitWindowSize(g_iWinWidth,g_iWinHeight);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);

  // main window
  g_iMainWindow=glutCreateWindow("GPUmesh OpenGL tutorial 1 (indexed mesh)");
  glutMouseFunc(mainMouse);
  glutMotionFunc(mainMotion);
  glutKeyboardFunc(mainKeyboard);
  glutDisplayFunc(mainRender);
  glutReshapeFunc(mainReshape);
  glutSetWindow(g_iMainWindow);
  glutIdleFunc(mainIdle);

  // gluX
  gluxInit();

  // create objects

  g_Object = new IndexedMesh();

  g_Object->begin(GPUMESH_TRIANGLELIST);

  g_Object->color0_4(255,255,255,0);
  g_Object->texcoord0_2(0.0,0.0);
  g_Object->texcoord1_2(0.0,0.0);
  g_Object->texcoord2_2(0.0,0.0);
  g_Object->texcoord3_2(0.0,0.0);
  g_Object->texcoord4_2(0.0,0.0);
  g_Object->texcoord5_2(0.0,0.0);
  g_Object->texcoord6_2(0.0,0.0);
  g_Object->texcoord7_2(0.0,0.0);
  g_Object->vertex_2(0.1,0.1);

  g_Object->color0_4(255,255,255,0);
  g_Object->texcoord0_2(0.0,1.0);
  g_Object->texcoord1_2(0.0,1.0);
  g_Object->texcoord2_2(0.0,1.0);
  g_Object->texcoord3_2(0.0,1.0);
  g_Object->texcoord4_2(0.0,1.0);
  g_Object->texcoord5_2(0.0,1.0);
  g_Object->texcoord6_2(0.0,1.0);
  g_Object->texcoord7_2(0.0,1.0);
  g_Object->vertex_2(0.1,0.9);

  g_Object->color0_4(255,255,255,0);
  g_Object->texcoord0_2(1.0,1.0);
  g_Object->texcoord1_2(1.0,1.0);
  g_Object->texcoord2_2(1.0,1.0);
  g_Object->texcoord3_2(1.0,1.0);
  g_Object->texcoord4_2(1.0,1.0);
  g_Object->texcoord5_2(1.0,1.0);
  g_Object->texcoord6_2(1.0,1.0);
  g_Object->texcoord7_2(1.0,1.0);
  g_Object->vertex_2(0.9,0.9);

  g_Object->color0_4(255,255,255,0);
  g_Object->texcoord0_2(1.0,0.0);
  g_Object->texcoord1_2(1.0,0.0);
  g_Object->texcoord2_2(1.0,0.0);
  g_Object->texcoord3_2(1.0,0.0);
  g_Object->texcoord4_2(1.0,0.0);
  g_Object->texcoord5_2(1.0,0.0);
  g_Object->texcoord6_2(1.0,0.0);
  g_Object->texcoord7_2(1.0,0.0);
  g_Object->vertex_2(0.9,0.1);

  g_Object->index(0);
  g_Object->index(1);
  g_Object->index(2);

  g_Object->index(0);
  g_Object->index(2);
  g_Object->index(3);

  g_Object->end();

  GLint v;
  glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB,&v);
  cerr << "Number of available texture units " 
       << v
       << endl << endl;
  g_iNbTU=min(v,8);
  // load textures
  for (int i=0;i<8;i++)
  {
    static char str[16];
    sprintf(str,"%d.tga",i);
    g_Texs[i]=gltex::gltexLoadTexture(str);
  }

  // help
  printf("[q]     - quit\n");
	
  // let's go
  glutMainLoop();

  return (0);
}
/* -------------------------------------------------------- */
